var H5P = H5P || {};

H5P.Caesar = (function ($) {

  /**
   * @class H5P.Caesar
   * @param {Object} params
   */
  function Caesar(options, id) {
    this.options = $.extend(true, {}, {
      alphabet: 'ABCDEFGHIJKLMNOPQRSTUVWXYZ',
      size: 300,
      fontsize: 24
    }, options);
    var alphabet = this.options.alphabet;
    var size = this.options.size;
    var fontsize = this.options.fontsize;
    var padding = 8;
    var outerRadius = size / 2;
    var middleRadius = size / 2 - (fontsize + padding);
    var innerRadius = size / 2 - 2 * (fontsize + padding);
    var cx = size / 2 + padding;
    var cy = size / 2 + padding;
    var angle = 0;

    this.attach = function (container) {
      container.css('height', size + padding * 2);
      container.css('width', size + padding * 2);
      container.svg({ onLoad: drawWheel, settings: { height: (size + padding * 2), width: (size + padding * 2) } });
      container.addClass('h5p-caesar');

    };

    function drawWheel(svg) {
      svg.circle(cx, cy, outerRadius, { class: 'h5p-caesar-wheel-outer h5p-caesar-wheel' });
      svg.circle(cx, cy, middleRadius, { class: 'h5p-caesar-wheel-middle h5p-caesar-wheel' });
      svg.circle(cx, cy, innerRadius, { class: 'h5p-caesar-wheel-inner h5p-caesar-wheel' });

      var gInner = svg.group(null, 'h5p-caesar-inner-wheel');
      var gOuter = svg.group(null, 'h5p-caesar-outer-wheel');

      for (i = 0; i < alphabet.length; i++) {
        let ix = cx + Math.cos(2 * Math.PI * i / alphabet.length) * innerRadius;
        let iy = cy + Math.sin(2 * Math.PI * i / alphabet.length) * innerRadius;
        let mx = cx + Math.cos(2 * Math.PI * i / alphabet.length) * middleRadius;
        let my = cy + Math.sin(2 * Math.PI * i / alphabet.length) * middleRadius;
        let ox = cx + Math.cos(2 * Math.PI * i / alphabet.length) * outerRadius;
        let oy = cy + Math.sin(2 * Math.PI * i / alphabet.length) * outerRadius;
        let tix = cx + Math.cos(2 * Math.PI * (i + 0.5) / alphabet.length) * (innerRadius + middleRadius - padding) / 2;
        let tiy = cy + Math.sin(2 * Math.PI * (i + 0.5) / alphabet.length) * (innerRadius + middleRadius - padding) / 2;
        let tox = cx + Math.cos(2 * Math.PI * (i + 0.5) / alphabet.length) * (outerRadius + middleRadius - padding) / 2;
        let toy = cy + Math.sin(2 * Math.PI * (i + 0.5) / alphabet.length) * (outerRadius + middleRadius - padding) / 2;
        svg.line(gInner, ix, iy, mx, my, { stroke: 'black', class: 'h5p-caesar-wheel-divider' });
        svg.line(gOuter, mx, my, ox, oy, { stroke: 'black', class: 'h5p-caesar-wheel-divider' });
        svg.text(gInner, tix, tiy, alphabet[i], { transform: 'rotate(' + ((360 * (i + 0.5)) / alphabet.length + 90) + ',' + tix + ',' + tiy + ')', 'dominant-baseline': 'middle', 'text-anchor': 'middle', class: 'h5p-caesar-inner-wheel-text h5p-caesar-wheel-text', style: 'font-size: ' + fontsize + 'px' });
        svg.text(gOuter, tox, toy, alphabet[i], { transform: 'rotate(' + ((360 * (i + 0.5)) / alphabet.length + 90) + ',' + tox + ',' + toy + ')', 'dominant-baseline': 'middle', 'text-anchor': 'middle', class: 'h5p-caesar-outer-wheel-text h5p-caesar-wheel-text', style: 'font-size: ' + fontsize + 'px' });
      }

      $('#h5p-caesar-inner-wheel').css('transform-origin', cx + 'px ' + cy + 'px');   
      
      svg.text(null, cx, cy, '', {'dominant-baseline': 'middle', 'text-anchor': 'middle', class: 'h5p-caesar-rotator-button'});
      
      $('.h5p-caesar-rotator-button').on('click', function () {
        angle += 360 / alphabet.length;
        $('#h5p-caesar-inner-wheel').css('transform', 'rotate(' + angle + 'deg)');
      });
    }
  }

  return Caesar;
})(H5P.jQuery);